'use strict';

var path = require('path');

const getByteSize = (content) => {
    if (typeof content === 'string') {
        return Buffer.from(content).length;
    }
    return content?.length || 0;
};
const bundleToWebpackStats = (bundle, customOptions) => {
    const options = {
        moduleOriginalSize: false,
        ...customOptions,
    };
    const items = Object.values(bundle);
    const assets = [];
    const chunks = [];
    const moduleByFileName = {};
    items.forEach(item => {
        if (item.type === 'chunk') {
            assets.push({
                name: item.fileName,
                size: getByteSize(item.code),
            });
            const chunkId = item.name;
            chunks.push({
                id: chunkId,
                entry: item.isEntry,
                initial: !item.isDynamicEntry,
                files: [item.fileName],
                names: [item.name],
            });
            Object.entries(item.modules).forEach(([modulePath, moduleInfo]) => {
                // Remove unexpected rollup null prefix
                const normalizedModulePath = modulePath.replace('\u0000', '');
                const relativeModulePath = path.relative(process.cwd(), normalizedModulePath);
                // Match webpack output - add current directory prefix for child modules
                const relativeModulePathWithPrefix = relativeModulePath.match(/^\.\./)
                    ? relativeModulePath
                    : `.${path.sep}${relativeModulePath}`;
                const moduleEntry = moduleByFileName[relativeModulePathWithPrefix];
                if (moduleEntry) {
                    moduleEntry.chunks.push(chunkId);
                }
                else {
                    moduleByFileName[relativeModulePathWithPrefix] = {
                        name: relativeModulePathWithPrefix,
                        size: options.moduleOriginalSize
                            ? moduleInfo.originalLength
                            : moduleInfo.renderedLength,
                        chunks: [chunkId],
                    };
                }
            });
        }
        else if (item.type === 'asset') {
            assets.push({
                name: item.fileName,
                size: getByteSize(item.source.toString()),
            });
        }
        else ;
    });
    return {
        builtAt: Date.now(),
        assets,
        chunks,
        modules: Object.values(moduleByFileName),
    };
};

const NAME = 'webpackStats';
const webpackStats = (options = {}) => ({
    name: NAME,
    generateBundle(_, bundle) {
        const output = bundleToWebpackStats(bundle, options);
        this.emitFile({
            type: 'asset',
            fileName: options?.fileName || 'webpack-stats.json',
            source: JSON.stringify(output),
        });
    },
});

exports.bundleToWebpackStats = bundleToWebpackStats;
exports.webpackStats = webpackStats;
//# sourceMappingURL=index.js.map
